/**
* @file CAL Code to Arabic Unicode conversion
* @version 1.0.7
* @author Greg Borota
* @copyright (c) 2017 Greg Borota.
* @license MIT
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

// https://peshitta.github.io
// https://sedra.bethmardutho.org/about/fonts
// http://cal1.cn.huc.edu/searching/fullbrowser.html

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('aramaic-mapper'), require('cal-code-util'), require('arabic-code-util')) :
	typeof define === 'function' && define.amd ? define(['exports', 'aramaic-mapper', 'cal-code-util', 'arabic-code-util'], factory) :
	(factory((global.calArabic = {}),global.aramaicMapper,global.calCodeUtil,global.arabicCodeUtil));
}(this, (function (exports,aramaicMapper,calCodeUtil,arabicCodeUtil) { 'use strict';

/** @module calArabic */
/**
 * @private
 * CAL source writing
 * @const
 * @type { Writing }
 */
var calWriting = new aramaicMapper.Writing(
  calCodeUtil.allConsonants,
  calCodeUtil.vowels,
  calCodeUtil.diacritics,
  calCodeUtil.punctuation
);

var feh = '\u0641'; // ف ARABIC LETTER FEH
var seen = '\u0633'; // س ARABIC LETTER SEEN

/**
 * @private
 * Arabic destination writing
 * @const
 * @type { Writing }
 */
var arabicWriting = new aramaicMapper.Writing(
  Object.freeze(arabicCodeUtil.consonants.concat(feh, seen)),
  Object.freeze([
    arabicCodeUtil.vowels[0], // a: fatha
    '', // o: callback will use superscript alef or alef maddah as needed
    arabicCodeUtil.vowels[2], // e: triple kasra to cover `e` and `i` and `E`
    arabicCodeUtil.vowels[2], // i
    arabicCodeUtil.vowels[3], // u: duplicate damma to cover both `u` and `O`
    arabicCodeUtil.vowels[2], // E
    arabicCodeUtil.vowels[3] // O:
  ]),
  Object.freeze(['', '', '', '']),
  Object.freeze([
    '\u061B', // ؛ Arabic Semicolon - also used with Thaana and Syriac in modern text
    '\u061F', // ؟ Arabic Question Mark - also used with Thaana and Syriac in modern text
    '\u0021', // ! Exclamation Mark - regular ASCII exclamation mark
    '.',
    '-',
    ':'
  ])
);

/**
 * @private
 * Maps input character to Arabic char
 * @param { string } c input character
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @returns { string } Arabic mapped char
 */
var to = function (c, fromTo) { return fromTo[c] || (fromTo[c] === '' ? '' : c); };
var alef = '\u0627'; // ا ARABIC LETTER ALEF
var fatha = '\u064E'; //  َ Arabic fatha - Garshuni: a
var supAlef = '\u0670'; //  ٰ Arabic letter superscript alef - Garshuni: long a
var kasra = '\u0650'; //  ِ Arabic kasra - Garshuni: i
var damma = '\u064F'; //  ُ Arabic damma - Garshuni: u
var waw = '\u0648'; // و ARABIC LETTER WAW
var yeh = '\u064A'; // ي ARABIC LETTER YEH
var madda = '\u0622'; // آ ARABIC LETTER ALEF WITH MADDA ABOVE
var ghain = '\u063A'; // غ ARABIC LETTER GHAIN
var thal = '\u0630'; // ذ ARABIC LETTER THAL
var khah = '\u062E'; // خ ARABIC LETTER KHAH
var theh = '\u062B'; // ث ARABIC LETTER THEH

/**
 * @private
 * Return true Y|w+vowel is flipped a la Sedra
 * @param { string } word word to test
 * @param { number } index position in word to start checking from
 * @returns { boolean } true if Y|w+vowel is flipped
 */
var isYwFlipped = function (word, index) {
  if (index < 1) {
    return false; // can't start word with a vowel
  }
  if (calCodeUtil.isVowel(word.charAt(index - 1))) {
    return false; // can't follow vowel by vowel
  }
  if (word.charAt(index - 1) === ')') {
    return index === 1 || calCodeUtil.isBdwlPrefix(word, index - 2); // initial Alap is followed by vowel
  }
  return true;
};

/**
 * @private
 * Maps input character to Arabic character
 * @param { string } c CAL input character
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @param { Object } wordProps optional word settings
 * @returns { string } Arabic mapped char
 */
var mapCallback = function (word, i, fromTo, wordProps) {
  var c = word.charAt(i);
  if (!wordProps.isDotted) {
    return to(c, fromTo);
  }
  var m = '';
  var n = word.charAt(i + 1);
  switch (c) {
    case 'g':
      m = n === ',' ? ghain : to(c, fromTo);
      break;
    case 'd':
      m = n === ',' ? thal : to(c, fromTo);
      break;
    case 'k':
      m = n === ',' ? khah : to(c, fromTo);
      break;
    case 't':
      m = n === ',' ? theh : to(c, fromTo);
      break;
    case 'y':
      m =
        n === 'i' || n === 'e'
          ? isYwFlipped(word, i) ? kasra + yeh : yeh + kasra
          : to(c, fromTo);
      break;
    case 'w':
      m =
        n === 'u' || n === 'O'
          ? isYwFlipped(word, i) ? damma + waw : waw + damma
          : to(c, fromTo);
      break;
    case ')':
      m = n === 'o' ? madda : alef;
      break;
    case 'o':
      // using alef subscript to avoid introducing new alef not found in source
      m =
        word.charAt(i - 1) === ')'
          ? '' // handled already as madda
          : n === ')' ? fatha : supAlef;
      break;
    default:
      m = to(c, fromTo);
      break;
  }
  return m;
};

/**
 * Arabic Mapper
 * @const
 * @type { Mapper }
 */
var mapper = new aramaicMapper.Mapper(calWriting, arabicWriting, mapCallback);

/**
 * Convert from CAL to Arabic
 * @static
 * @param {string} word input word in CAL code transliteration
 * @returns {string} the input word converted to Arabic Unicode
 */
var toArabic = function (word) {
  var wordProp = Object.freeze(
    Object.create(null, {
      isDotted: { value: calCodeUtil.isDotted(word), enumerable: true }
    })
  );
  return mapper.map(word, wordProp);
};

exports.mapper = mapper;
exports.toArabic = toArabic;

Object.defineProperty(exports, '__esModule', { value: true });

})));
